#ifndef _LIBMRTD_H
#define _LIBMRTD_H

#include <unistd.h>
#include <sys/types.h>

#include <openssl/des.h>

#ifdef _HAVE_OPENCT
#include <openct/openct.h>
#endif

#ifdef _HAVE_PCSC
#include <winscard.h>
#include <pcsclite.h>
#endif

#ifdef _HAVE_LIBRFID
#include <rfid/rfid_protocol.h>
#include <rfid/rfid_layer2.h>
#include <rfid/rfid_reader.h>
#endif

enum icao_lds_fid {
	ICAO_LDS_FID_COM 	= 0x011e,
	ICAO_LDS_FID_DG1	= 0x0101,
	ICAO_LDS_FID_DG2	= 0x0102,
	ICAO_LDS_FID_DG3	= 0x0103,
	ICAO_LDS_FID_DG4	= 0x0104,
	ICAO_LDS_FID_DG5	= 0x0105,
	ICAO_LDS_FID_DG6	= 0x0106,
	ICAO_LDS_FID_DG7	= 0x0107,
	ICAO_LDS_FID_DG8	= 0x0108,
	ICAO_LDS_FID_DG9	= 0x0109,
	ICAO_LDS_FID_DG10 	= 0x010a,
	ICAO_LDS_FID_DG11 	= 0x010b,
	ICAO_LDS_FID_DG12 	= 0x010c,
	ICAO_LDS_FID_DG13 	= 0x010d,
	ICAO_LDS_FID_DG14 	= 0x010e,
	ICAO_LDS_FID_DG15 	= 0x010f,
	ICAO_LDS_FID_DG16 	= 0x0110,
	ICAO_LDS_FID_SOD	= 0x011d,
};

/* 88 bytes of MRZ information */
struct mrtd_mrz {
	char	doc_type[2];
	char	country[3];
	char	name[39];
	char	doc_nr[9];
	char	doc_nr_check;
	char	nationality[3];
	char	date_of_birth[6];
	char	dob_check[1];
	char	_unknown1[1];
	char	date_of_expiry[6];
	char	_unknown2[1];
	char	_unknown3[16];
} __attribute__ ((packed));

#define	KEY_LEN_TDES	16

/* SM state of an encrypted session */
struct session_state {
	u_int64_t		ssc;		/* send sequence counter */

	char		ks_enc[KEY_LEN_TDES];
	DES_key_schedule	ks_enc_ks1;
	DES_key_schedule	ks_enc_ks2;

	char		ks_mac[KEY_LEN_TDES];	/* session 3des keys */
	DES_key_schedule	ks_mac_ks1;
	DES_key_schedule	ks_mac_ks2;
};

struct mrtd_handle {
	struct {
		int (*transcieve)(struct mrtd_handle *h,
				  const unsigned char *sbuf, unsigned int slen,
				  unsigned char *rbuf, unsigned int *rlen);
	} fn;
	struct {
		struct session_state sstate;
		int (*transcieve)(struct mrtd_handle *h,
				  const unsigned char *sbuf, unsigned int slen,
				  unsigned char *rbuf, unsigned int *rlen);
	} bac;
	union {
#ifdef _HAVE_PCSC
		struct {
			SCARDCONTEXT ctx;
		} pcsc;
#endif
#ifdef _HAVE_OPENCT
		struct {
			ct_handle *handle;			
			int slot;
		} openct;
#endif
#ifdef _HAVE_LIBRFID
		struct {
			struct rfid_reader_handle *rh;
			struct rfid_layer2_handle *l2h;
			struct rfid_protocol_handle *ph;
		} librfid;
#endif
	} transp;

	u_int16_t dgpresent;
};

extern const char *mrtd_hexdump(const void *data, unsigned int len);
extern int read_mrtd(struct mrtd_handle *ih, char *mrz2);

struct mrtd_bio_header_tmpl {
	u_int16_t	hdr_version;
	u_int8_t	bio_type;
	u_int8_t	bio_feature;
	time_t		creation_time;
	time_t		valid_from;
	time_t		valid_through;
	u_int16_t	creator_pid;
	u_int16_t	format_owner;
	u_int16_t	format_type;
};

struct mrtd_bio_tmpl {
	struct mrtd_bio_header_tmpl bht;
	int encrypted;
	unsigned char *bio_data;
	int bio_data_len;
};


#endif

