#ifndef _MRTD_H

#include <unistd.h>
#include <sys/types.h>

#include <openssl/des.h>

#ifdef _HAVE_OPENCT
#include <openct/openct.h>
#endif

#ifdef _HAVE_PCSC
#include <winscard.h>
#include <pcsclite.h>
#endif

#ifdef _HAVE_LIBRFID
#include <librfid/rfid_protocol.h>
#include <librfid/rfid_layer2.h>
#include <librfid/rfid_reader.h>
#endif

enum icao_lds_fid {
	ICAO_LDS_FID_COM 	= 0x011e,
	ICAO_LDS_FID_DG1	= 0x0101,
	ICAO_LDS_FID_DG2	= 0x0102,
	ICAO_LDS_FID_DG3	= 0x0103,
	ICAO_LDS_FID_DG4	= 0x0104,
	ICAO_LDS_FID_DG5	= 0x0105,
	ICAO_LDS_FID_DG6	= 0x0106,
	ICAO_LDS_FID_DG7	= 0x0107,
	ICAO_LDS_FID_DG8	= 0x0108,
	ICAO_LDS_FID_DG9	= 0x0109,
	ICAO_LDS_FID_DG10 	= 0x010a,
	ICAO_LDS_FID_DG11 	= 0x010b,
	ICAO_LDS_FID_DG12 	= 0x010c,
	ICAO_LDS_FID_DG13 	= 0x010d,
	ICAO_LDS_FID_DG14 	= 0x010e,
	ICAO_LDS_FID_DG15 	= 0x010f,
	ICAO_LDS_FID_DG16 	= 0x0110,
	ICAO_LDS_FID_SOD	= 0x011d,
};


#define	KEY_LEN_TDES	16

/* SM state of an encrypted session */
struct session_state {
	u_int64_t		ssc;		/* send sequence counter */

	char		ks_enc[KEY_LEN_TDES];
	DES_key_schedule	ks_enc_ks1;
	DES_key_schedule	ks_enc_ks2;

	char		ks_mac[KEY_LEN_TDES];	/* session 3des keys */
	DES_key_schedule	ks_mac_ks1;
	DES_key_schedule	ks_mac_ks2;
};

struct mrtd_ef {
	unsigned char *data;
	unsigned int len;
};

struct mrtd_handle {
	struct {
		int (*transceive)(struct mrtd_handle *h,
				  const char *sbuf, unsigned int slen,
				  char *rbuf, unsigned int *rlen);
	} fn;
	struct {
		struct session_state sstate;
		int (*transceive)(struct mrtd_handle *h,
				  const char *sbuf, unsigned int slen,
				  char *rbuf, unsigned int *rlen);
	} bac;
	union {
#ifdef _HAVE_PCSC
		struct {
			SCARDCONTEXT ctx;
		} pcsc;
#endif
#ifdef _HAVE_OPENCT
		struct {
			ct_handle *handle;			
			int slot;
		} openct;
#endif
#ifdef _HAVE_LIBRFID
		struct {
			struct rfid_reader_handle *rh;
			struct rfid_protocol_handle *l2h;
			struct rfid_protocol_handle *ph;
		} librfid;
#endif
	} transp;

	u_int16_t dgpresent;

	struct mrtd_ef ef[16];

	struct iso19794_5_info *dg2_info;
};


#endif /* _MRTD_H */
