#ifndef _ISO19794_5_H
#define _ISO19794_5_H

/* 14 byte Facial Record Header */
struct iso19794_5_facial_record_hdr {
	char		format_id[4];
	char		version[4];
	u_int32_t	length;		/* includes record header */
	u_int16_t	num_images;
} __attribute ((packed));

#define ISO19794_5_FORMAT_IDENTIFIER	"FAC"
#define ISO19794_5_VERSION		"010"

struct iso19794_5_pose_angle {
	u_int8_t	y;
	u_int8_t	p;
	u_int8_t	r;
} __attribute__ ((packed));

/* 20 byte Facial Information Block */
struct iso19794_5_facial_info_block {
	u_int32_t			fi_block_length;
	u_int16_t			num_feature_points;
	u_int8_t			gender;
	u_int8_t			eye_colour;
	u_int8_t			hair_colour;
	char				feature_mask[3];
	u_int16_t			expression;
	struct iso19794_5_pose_angle	pose_angle;
	struct iso19794_5_pose_angle	pose_angle_uncertainty;
} __attribute__ ((packed));

enum {
	ISO19794_5_GENDER_UNSPEC	= 0,
	ISO19794_5_GENDER_MALE		= 1,
	ISO19794_5_GENDER_FEMALE	= 2,
	ISO19794_5_GENDER_UNKNOWN	= 3,
};

enum {
	ISO19794_5_EYECOLOUR_UNPEC	= 0x00,
	ISO19794_5_EYECOLOUR_BLACK	= 0x01,
	ISO19794_5_EYECOLOUR_BLUE	= 0x02,
	ISO19794_5_EYECOLOUR_BROWN	= 0x03,
	ISO19794_5_EYECOLOUR_GRAY	= 0x04,
	ISO19794_5_EYECOLOUR_GREEN	= 0x05,
	ISO19794_5_EYECOLOUR_MULTI	= 0x06,
	ISO19794_5_EYECOLOUR_PINK	= 0x07,
	ISO19794_5_EYECOLOUR_OTHER	= 0x08,
};

enum {
	ISO19794_5_HAIRCOLOUR_UNSPEC	= 0x00,
	ISO19794_5_HAIRCOLOUR_BALD	= 0x01,
	ISO19794_5_HAIRCOLOUR_BLACK	= 0x02,
	ISO19794_5_HAIRCOLOUR_BLONDE	= 0x03,
	ISO19794_5_HAIRCOLOUR_BROWN	= 0x04,
	ISO19794_5_HAIRCOLOUR_GRAY	= 0x05,
	ISO19794_5_HAIRCOLOUR_WHITE	= 0x06,
	ISO19794_5_HAIRCOLOUR_RED	= 0x07,
	ISO19794_5_HAIRCOLOUR_GREEN	= 0x08,
	ISO19794_5_HAIRCOLOUR_BLUE	= 0x09,
	ISO19794_5_HAIRCOLOUR_OTHER	= 0xff,
};

enum {
	ISO19794_5_FEATURE_SPECIFIED	= (1 << 0),
	ISO19794_5_FEATURE_GLASSES	= (1 << 1),
	ISO19794_5_FEATURE_MOUSTACHE	= (1 << 2),
	ISO19794_5_FEATURE_BEARD	= (1 << 3),
	ISO19794_5_FEATURE_TEETH_VIS	= (1 << 4),
	ISO19794_5_FEATURE_BLINK	= (1 << 5),
	ISO19794_5_FEATURE_MOUTH_OPEN	= (1 << 6),
	ISO19794_5_FEATURE_LEFT_EYEPATCH= (1 << 7),
	ISO19794_5_FEATURE_RIGHT_EYEPATCH= (1 << 8),
	ISO19794_5_FEATURE_DARK_GLASSES	= (1 << 9),
	ISO19794_5_FEATURE_DISTORTED	= (1 << 10),
};

enum {
	ISO19794_5_EXPR_UNSPEC		= 0x0000,
	ISO19794_5_EXPR_NEUTRAL		= 0x0001,
	ISO19794_5_EXPR_SMILE_CLOSED	= 0x0002,
	ISO19794_5_EXPR_SMILE_OPEN	= 0x0003,
	ISO19794_5_EXPR_RAISED_EYEBROW	= 0x0004,
	ISO19794_5_EXPR_LOOKING_AWAY	= 0x0005,
	ISO19794_5_EXPR_SQUINTING	= 0x0006,
	ISO19794_5_EXPR_FROWNING	= 0x0007,
};
#define ISO19794_5_EXPR_VENDOR_MASK	0x1000


/* Optional 8 byte Feature Point Block */
struct iso19794_5_feature_point {
	u_int8_t	type;
	u_int8_t	point;	/* According to ISO/IEC 14496-2 */
	u_int16_t	x;
	u_int16_t	y;
	u_int16_t	reserved;
} __attribute__ ((packed));

#define ISO19794_5_FP_TYPE_2D	0x01

#define ISO19794_5_FP_POINT_A(x)	(x >> 4)
#define ISO19794_5_FP_POINT_B(x)	(x & 0x0f)

/* 12 Byte Image Information Block */
struct iso19794_5_image_info {
	u_int8_t	image_type;
	u_int8_t	data_type;
	u_int16_t	width;
	u_int16_t	height;
	u_int8_t	cspace;
	u_int8_t	source_type;
	u_int16_t	device_type;
	u_int16_t	quality;

};

enum {
	ISO19794_5_IMG_TYPE_UNSPEC	= 0x00,
	ISO19794_5_IMG_TYPE_BASIC	= 0x01,
	ISO19794_5_IMG_TYPE_FULL_FRONT	= 0x02,
	ISO19794_5_IMG_TYPE_TOKEN_FRONT	= 0x03,
	ISO19794_5_IMG_TYPE_OTHER	= 0x04,
};

enum {
	ISO19794_5_IMG_DTYPE_JPEG	= 0x00,
	ISO19794_5_IMG_DTYPE_JPEG2000	= 0x01,
};

enum {
	ISO19794_5_IMG_CSPACE_UNSPEC	= 0x00,
	ISO19794_5_IMG_CSPACE_RGB24	= 0x01,
	ISO19794_5_IMG_CSPACE_YUV422	= 0x02,
	ISO19794_5_IMG_CSPACE_GREY8BIT	= 0x03,
	ISO19794_5_IMG_CSPACE_OTHER	= 0x04,
};

enum {
	ISO19794_5_IMG_SOURCE_UNSPEC		= 0x00,
	ISO19794_5_IMG_SOURCE_STATIC_UNSPEC	= 0x01,
	ISO19794_5_IMG_SOURCE_STATIC_DIGITAL	= 0x02,
	ISO19794_5_IMG_SOURCE_STATIC_SCAN	= 0x03,
	ISO19794_5_IMG_SOURCE_VIDEO_UKNKOWN	= 0x04,
	ISO19794_5_IMG_SOURCE_VIDEO_ANALOG	= 0x05,
	ISO19794_5_IMG_SOURCE_VIDEO_DIGITAL	= 0x06,
	ISO19794_5_IMG_SOURCE_UNKNOWN		= 0x07,
};


/* non-official data structures specific to this implementation */

struct mrtd_facial_record {
	struct iso19794_5_facial_info_block 	facial_info;
	struct iso19794_5_image_info		image_info;
	void 					*image_data;
	unsigned int				image_data_len;
	struct iso19794_5_feature_point		**feature_point;
};

struct iso19794_5_info {
	struct iso19794_5_facial_record_hdr frh;
	struct mrtd_facial_record fr[0];
};

extern struct iso19794_5_info *
iso19794_5_parse(const char *data, unsigned int len);

extern char *iso19794_5_gender(u_int8_t gender);
extern char *iso19794_5_eye_colour(u_int8_t colour);
extern char *iso19794_5_hair_colour(u_int8_t colour);
extern char *iso19794_5_features(u_int32_t features);
extern char *iso19794_5_expression(u_int16_t expr);

extern char *iso19794_5_img_type(u_int8_t type);
extern char *iso19794_5_img_data_type(u_int8_t dtype);
extern char *iso19794_5_img_cspace(u_int8_t cspace);
extern char *iso19794_5_img_source(u_int8_t source);

#endif
