/* ISO7816-4 helper functions for libicao/libmtrd
 *
 * (C) 2005 by Harald Welte <hwelte@hmw-consulting.de>
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <netinet/in.h>

#include <libmrtd/icao.h>
#include <libmrtd/libmrtd.h>

int
iso7816_get_challenge(struct mrtd_handle *h, 
		      unsigned char *challenge, unsigned int len)
{
	unsigned char cmd[5] = { 0x00, 0x84, 0x00, 0x00, 0x00 };
	unsigned char retbuf[256+2];
	unsigned int retlen = sizeof(retbuf);
	int rv;

	if (len > 0xff)
		return -1;

	cmd[4] = len;

	rv = h->fn.transceive(h, cmd, 5, retbuf, &retlen);
	if (rv < 0)
		return rv;

	printf("%d [%s]\n", retlen, mrtd_hexdump(retbuf, retlen));

	if (retlen <= 2)
		return -EINVAL;

	memcpy(challenge, retbuf, retlen-2);	/* FIXME: ? */

	return 0;
}

int
iso7816_select_application(struct mrtd_handle *ih)
{
	unsigned char cmd[] = { 0x00, 0xa4, 0x04, 0x0c, 0x07,
		       0xa0, 0x00, 0x00, 0x02, 0x47, 0x10, 0x01 };
	unsigned char resp[70];
	unsigned int rlen = sizeof(resp);

	int rv;

	//rv = ih->fn.transcieve(ih, cmd, sizeof(cmd), resp, &rlen);
	rv = ih->fn.transceive(ih, cmd, sizeof(cmd), (char *)resp, &rlen);
	if (rv < 0)
		return rv;

	if (rlen != 2 || resp[0] != 0x90 || (resp[1] & 0xff) != 0x00)
		return -1;

	return 0;
}

int
iso7816_select_ef(struct mrtd_handle *ih, u_int16_t fid)
{
	unsigned char cmd[7] = { 0x00, 0xa4, 0x02, 0x0c, 0x02, 0x00, 0x00 };
	unsigned char resp[70];
	unsigned int rlen = sizeof(resp);

	int rv;

	cmd[5] = (fid >> 8) & 0xff;
	cmd[6] = fid & 0xff;

	rv = ih->fn.transceive(ih, cmd, sizeof(cmd), resp, &rlen);
	if (rv < 0)
		return rv;

	if (rlen < 2 || resp[rlen-2] != 0x90 || resp[rlen-1] != 0x00)
		return -1;

	return 0;
}

int
iso7816_read_binary(struct mrtd_handle *ih, 
		    unsigned char *buf, unsigned int *len)
{
	unsigned char cmd[] = { 0x00, 0xb0, 0x00, 0x00, 0x00 };
	unsigned char resp[256+2];
	unsigned int rlen;

	u_int16_t already_read = 0;
	
	int rv;

	do {
		unsigned int bytes_to_read = *len - already_read;
		if (bytes_to_read == 0)
			return 0;

		if (bytes_to_read <= 255)
			cmd[4] = bytes_to_read;

		*((u_int16_t *)(cmd+2)) = htons(already_read);

		rlen = sizeof(resp);
		rv = ih->fn.transceive(ih, cmd, sizeof(cmd), (char *)resp, &rlen);
		if (rv < 0)
			return rv;
	
		if (rlen < 2)
			return -EINVAL;

		if (rlen > bytes_to_read+2)

		if (resp[rlen-2] != 0x90 || resp[rlen-1] != 0x00) {
			printf("SW1=%x SW2=%x\n", resp[rlen-2], resp[rlen-1]);
			return -EINVAL;
		}

		if (rlen == 2)
			break;

		memcpy(buf+already_read, resp, rlen-2);

		already_read += rlen-2;
		/* FIXME: determine whether we need additional reads */
		printf("already_read = %u\n", already_read);
	} while (1);

	*len = already_read;

	printf("[%d] %s\n", already_read, mrtd_hexdump(buf, already_read));
	return 0;
}

/* wrapper function around SELECT EF and READ BINARY */
int
iso7816_read_ef(struct mrtd_handle *ih, u_int16_t fid,
		unsigned char *buf, unsigned int *len)
{
	int rv;

	printf("..selecting\n");
	rv = iso7816_select_ef(ih, fid);
	if (rv < 0)
		return rv;

	printf("..reading\n");
	return iso7816_read_binary(ih, buf, len);
}

