/* libicao - communication with ICAO compliant MRTD's 
 *
 * (C) 2005 by Harald Welte <hwelte@hmw-consulting.de>
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <libmrtd/icao.h>
#include <libmrtd/libmrtd.h>
#include <libmrtd/iso19794_5.h>

#include "iso7816_util.h"
#include "tlv.h"
#include "parse_display.h"
#include "basic_access.h"

const char *
mrtd_hexdump(const void *data, unsigned int len)
{
	static char string[1024];
	unsigned char *d = (unsigned char *) data;
	unsigned int i, left;

	memset(string, 0, sizeof(string));
	left = sizeof(string);
	for (i = 0; len--; i += 3) {
		if (i >= sizeof(string) -4)
			break;
		snprintf(string+i, 4, " %02x", *d++);
	}
	return string;
}


/* read out all data from a MRTD */
int 
read_mrtd(struct mrtd_handle *ih, char *mrz2)
{
	unsigned char ef_sod[32*1024];
	unsigned int ef_sod_len = sizeof(ef_sod);
	unsigned char ef_dg1[256];
	unsigned int ef_dg1_len = sizeof(ef_dg1);
	unsigned char ef_dg2[128*1024];
	unsigned int ef_dg2_len = sizeof(ef_dg2);
	u_int16_t dg_present;
	int rv;
	struct mrtd_bio_tmpl bt;

	ih->ef[0].len = 256;
	ih->ef[1].len = 256;
	ih->ef[2].len = 128*1024;

	ih->ef[0].data = malloc(ih->ef[0].len);
	ih->ef[1].data = malloc(ih->ef[1].len);
	ih->ef[2].data = malloc(ih->ef[2].len);

	if (!ih->ef[0].data || !ih->ef[1].data || !ih->ef[2].data)
		return -ENOMEM;

	rv = iso7816_select_application(ih);
	if (rv < 0)
		return rv;

	/* Basic Authentication */
	if (mrz2) {
		printf("running basic access control\n");
		int foo = basic_authenticate(ih, mrz2);
		if (foo < 0)
			return foo;
	}

	printf("selecting/reading EF.COM\n");
	rv = iso7816_read_ef(ih, ICAO_LDS_FID_COM, ih->ef[0].data, &ih->ef[0].len);
	if (rv < 0)
		return rv;

	printf("parsing EF.COM\n");
	/* FIXME: parse EF.COM, determine list of available DG's */
	rv = parse_ef_com(ih->ef[0].data, ih->ef[0].len, &dg_present);
	if (rv < 0)
		return rv;

	/* DG1 has to exist */
	if (!(dg_present & 0x0001)) {
		fprintf(stderr, "DG1 doesn't exist, aborting\n");
		return -EINVAL;
	}


	printf("selecting EF.DG1\n");
	rv = iso7816_read_ef(ih, ICAO_LDS_FID_DG1, ih->ef[1].data, &ih->ef[1].len);
	if (rv < 0)
		return rv;

	printf("parsing EF.DG1\n");
	rv = parse_ef_dg1(ih->ef[1].data, ih->ef[1].len);
	if (rv < 0)
		return rv;
	printf("%s\n", ih->ef[1].data);

#if 0
	printf("selecting/reading EF.SOD\n");
	/* parse EF.SOD and FIXME: verify EF.DG1 hash value */
	rv = iso7816_read_ef(ih, ICAO_LDS_FID_SOD, ef_sod, &ef_sod_len);
	if (rv < 0)
		return rv;
#endif

	/* DG2 has to exist */
	if (!(dg_present & 0x0002)) {
		fprintf(stderr, "DG2 doesn't exist, aborting\n");
		return -EINVAL;
	}

	printf("selecting/reading EF.DG2\n");
	rv = iso7816_read_ef(ih, ICAO_LDS_FID_DG2, ih->ef[2].data, &ih->ef[2].len);
	if (rv < 0)
		return rv;


	rv = parse_ef_dg234(ih->ef[2].data, 2, ih->ef[2].len, &bt);
	if (rv < 0)
		return rv;

	ih->dg2_info = iso19794_5_parse(bt.bio_data, bt.bio_data_len);
	if (!ih->dg2_info)
		return -1;

	//display_dg2(dg2_info);

	return 0;

}
