#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>

#include <libmrtd/libmrtd.h>

#include "tlv.h"

/* LDS 1.7, Table A1, Page 45 */

struct dg_param {
	int		group;
	char 		*name;
	char		sfid;
	u_int16_t	fid;
	char		tag;
};

static struct dg_param dg_params[] = {
{              .name =  "COM", .sfid = 0x1e, .fid = 0x011e, .tag = 0x60, },
{ .group =  1, .name =  "DG1", .sfid = 0x01, .fid = 0x0101, .tag = 0x61, },
{ .group =  2, .name =  "DG2", .sfid = 0x02, .fid = 0x0102, .tag = 0x75, },
{ .group =  3, .name =  "DG3", .sfid = 0x03, .fid = 0x0103, .tag = 0x63, },
{ .group =  4, .name =  "DG4", .sfid = 0x04, .fid = 0x0104, .tag = 0x76, },
{ .group =  5, .name =  "DG5", .sfid = 0x05, .fid = 0x0105, .tag = 0x65, },
{ .group =  6, .name =  "DG6", .sfid = 0x06, .fid = 0x0106, .tag = 0x66, },
{ .group =  7, .name =  "DG7", .sfid = 0x07, .fid = 0x0107, .tag = 0x67, },
{ .group =  8, .name =  "DG8", .sfid = 0x08, .fid = 0x0108, .tag = 0x68, },
{ .group =  9, .name =  "DG9", .sfid = 0x09, .fid = 0x0109, .tag = 0x69, },
{ .group = 10, .name = "DG10", .sfid = 0x0a, .fid = 0x010a, .tag = 0x6a, },
{ .group = 11, .name = "DG11", .sfid = 0x0b, .fid = 0x010b, .tag = 0x6b, },
{ .group = 12, .name = "DG12", .sfid = 0x0c, .fid = 0x010c, .tag = 0x6d, },
{ .group = 13, .name = "DG13", .sfid = 0x0d, .fid = 0x010d, .tag = 0x6e, },
{ .group = 14, .name = "DG14", .sfid = 0x0e, .fid = 0x010e, .tag = 0x6f, },
{ .group = 15, .name = "DG15", .sfid = 0x0f, .fid = 0x010f, .tag = 0x70, },
{ .group = 16, .name = "DG16", .sfid = 0x10, .fid = 0x0110, .tag = 0x75, },
{              .name =  "SOD", .sfid = 0x1d, .fid = 0x011d, .tag = 0x77, },
{ 0 } 
};


static u_int16_t
parse_dgpresent(unsigned char *list, int len)
{
	u_int16_t ret = 0;
	int i;

	for (i = 0; i < len; i++) {
		char tag = *(list+i);
		int j;

		for (j=0; j < sizeof(dg_params)/sizeof(struct dg_param); j++) {
			if (dg_params[j].tag == tag
			    && dg_params[j].group) {
				ret |= 1 << (dg_params[j].group - 1);
				break;
			}
		}
	}
	return ret;
}
		

int
parse_ef_com(unsigned char *ef_com, unsigned int len, u_int16_t *dg_present)
{
	int rc;
	unsigned char *cur = ef_com;
	struct tlv t, t2;

	rc = tlv_parse(ef_com, len, &t);
	if (rc < 0)
		return rc;

	if (!tlv_raw_tag(&t) == 0x60)
		return -EINVAL;

	if (tlv_gross_len(&t) > len)
		return -EINVAL;

	/* iterate over sub-TLVs */
	cur = t.val; 
	do {
		rc = tlv_parse(cur, t.len - (cur - t.val), &t2);
		if (rc < 0)
			return rc;
	     
		switch (tlv_raw_tag(&t2)) {
		case 0x5f01:
			/* LDS Version Number */
			break;
		case 0x5f36:
			/* Unicode Version Number */
			break;
		case 0x5c:
			/* DG's present */
			*dg_present = parse_dgpresent(t.val, t.len);
			break;
		default:
			break;
		}
	} while ((cur + tlv_gross_len(&t2) < t.val+t.len) && 
		 (cur += tlv_gross_len(&t2)));

	return 0;
}

struct dg1 {
	char	doc_type[2];
	char	country[3];
	char	name[39];
};
	
int
parse_ef_dg1(unsigned char *dg1, unsigned int len)
{
	int rc;
	struct tlv t, t2;
	unsigned char *cur;

	rc = tlv_parse(dg1, len, &t);
	if (rc < 0)
		return rc;

	if (tlv_raw_tag(&t) != 0x61)
		return -EINVAL;

	if (tlv_gross_len(&t) > len)
		return -EINVAL;

	cur = t.val;
	do {
		rc = tlv_parse(cur, t.len - (cur - t.val), &t2);
		if (rc < 0)
			return rc;

		switch (tlv_raw_tag(&t2)) {
		case 0x5f1f:
			/* MRZdata object */
			printf("MRZdata: %s\n", t2.val); //mrtd_hexdump(t2.val, t2.len));
			break;
		default:
			/* unknown object */
			printf("Unknown object 0x%x\n", tlv_raw_tag(&t2));
			break;
		}
	} while ((cur + tlv_gross_len(&t2) < t.val+t.len) && 
		 (cur += tlv_gross_len(&t2)));

	return 0;
}


static int 
parse_bht(struct tlv *t, struct mrtd_bio_header_tmpl *bht)
{
	int rc;
	struct tlv t2;

	for (rc = tlv_parse_first_child(t, &t2);
	     rc >= 0;
	     rc = tlv_parse_next_child(t, &t2, &t2)) {

		switch (tlv_raw_tag(&t2)) {
		case 0x80:
			bht->hdr_version = *(u_int16_t *)t2.val;
			break;
		case 0x81:
			bht->bio_type = *(t2.val);
			break;
		case 0x82:
			bht->bio_feature = *(t2.val);
			break;
		case 0x83:
			break;
		case 0x84:
			break;
		case 0x86:
			bht->creator_pid = *((u_int16_t *)t2.val);
			break;
		case 0x87:
			bht->format_owner = *((u_int16_t *)t2.val);
			break;
		case 0x88:
			bht->format_type = *((u_int16_t *)t2.val);
			break;
		}
	}

	return 0;
}

int
parse_biometric_template(const struct tlv *t, struct mrtd_bio_tmpl *bt)
{
	int rc;
	struct tlv t2;

	rc = tlv_parse_first_child(t, &t2);
	if (rc < 0)
		return rc;

	if (tlv_raw_tag(&t2) != 0xa1) /* FIXME: A2, ... */
		return -EINVAL;

	rc = parse_bht(&t2, &bt->bht);
	if (rc < 0)
		return rc;

	rc = tlv_parse_next_child(t, &t2, &t2);
	if (rc < 0)
		return rc;

	if (tlv_raw_tag(&t2) != 0x5f2e
	    && tlv_raw_tag(&t2) != 0x7f2e)
		return -EINVAL;

	bt->bio_data = t2.val;
	bt->bio_data_len = t2.len;

	return 0;
}


int
parse_ef_dg234(unsigned char *dg, unsigned int num,
	       unsigned int len, struct mrtd_bio_tmpl *bt)
{
	int rc;
	u_int8_t num_instances;
	struct tlv t, t2, t3;

	if (num < 2 || num > 4)
		return -EINVAL;

	rc = tlv_parse(dg, len, &t);
	if (rc < 0)
		return rc;

	if (tlv_raw_tag(&t) != dg_params[num].tag)
		return -EINVAL;

	if (tlv_gross_len(&t) > len)
		return -EINVAL;

	rc = tlv_parse_first_child(&t, &t2);
	if (rc < 0)
		return rc;

	/* only 0x7f61 template permitted at this location */
	if (tlv_raw_tag(&t2) != 0x7f61)
		return -EINVAL;

	if (tlv_gross_len(&t2) > t.len)
		return -EINVAL;

	/* reuse same tlv parser structure */
	rc = tlv_parse_first_child(&t2, &t3);
	if (rc < 0)
		return rc;

	/* only allow 0x02 - number of instances of this type of biometric */
	if (tlv_raw_tag(&t3) != 0x02)
		return -EINVAL;

	num_instances = *(t3.val);

	rc = tlv_parse_next_child(&t2, &t3, &t3);
	if (rc < 0)
		return -EINVAL;

	if (tlv_raw_tag(&t3) != 0x7f60)
		return -EINVAL;

	rc = parse_biometric_template(&t3, bt);
	if (rc < 0)
		return rc;

	/* FIXME */
	return 0;
}


