/* libmtrd OpenCT backend */
#ifdef _HAVE_LIBRFID

#include <errno.h>

#include <libmrtd/icao.h>
#include <libmrtd/libmrtd.h>

#include <librfid/rfid.h>
#include <librfid/rfid_reader.h>
#include <librfid/rfid_layer2.h>
#include <librfid/rfid_protocol.h>

static int
transceive_librfid(struct mrtd_handle *h,
		   const unsigned char *cmd, unsigned int len,
		   unsigned char *resp, unsigned int *rlen)
{
	int rv;
	
	rv = rfid_protocol_transceive(h->transp.librfid.ph, 
			      	      cmd, len, resp, rlen, 0, 0);
	if (rv < 0) {
		fprintf(stderr, "error\n");
		return -1;
	}

	return 0;
}

int 
mrtd_librfid_init(struct mrtd_handle *h)
{
	h->fn.transceive = transceive_librfid;

	rfid_init();

	h->transp.librfid.rh = rfid_reader_open(NULL, RFID_READER_CM5121);
	if (!h->transp.librfid.rh) {
		h->transp.librfid.rh = rfid_reader_open(NULL, RFID_READER_OPENPCD);
	}

	if (!h->transp.librfid.rh)
		return -EINVAL;

	h->transp.librfid.l2h = rfid_layer2_init(h->transp.librfid.rh, 
						 RFID_LAYER2_ISO14443A);
	if (!h->transp.librfid.l2h)
		return -EINVAL;

	if (rfid_layer2_open(h->transp.librfid.l2h) < 0)
		return -EINVAL;

	h->transp.librfid.ph = rfid_protocol_init(h->transp.librfid.l2h, 
						  RFID_PROTOCOL_TCL);
	if (!h->transp.librfid.ph)
		return -EINVAL;

	return rfid_protocol_open(h->transp.librfid.ph);
}

#endif
